package com.inspect.client.mapper;

import com.inspect.client.domain.ReqDomain;
import com.inspect.client.domain.ResDomain;
import org.apache.ibatis.annotations.Mapper;

@Mapper
public interface ClientStatisticsMapper {
   ResDomain queryTaskDone(ReqDomain var1);

   ResDomain queryPersonExamine(ReqDomain var1);

   ResDomain queryWarning(ReqDomain var1);

   ResDomain queryResultPersonExamine(ReqDomain var1);

   ResDomain queryPointNum(ReqDomain var1);

   String queryIICSystemCode();

   String queryPatrolSystemCodeByDeviceCode(String var1);

   String queryRobotSystemCode();

   String queryNestSystemCode();

   String selectStationNo();
}
